/*
 * A simple program to display and update a sprite using key strokes
 * Uses getKeyStates() method on a GameCanvas object
 */

package gamelibrary.game;

/**
 *
 * @author Griff
 */
import javax.microedition.lcdui.*;
import javax.microedition.lcdui.game.*;
import java.util.Vector;
import gameactual.*;
import gamelibrary.gameobject.*;
import gamelibrary.collisiondetection.*;

public class MyCanvas extends GameCanvas implements Runnable{
    private boolean running = true;
    private Coordinates viewCoords;
    private Coordinates maxScreenCoords;

    private Thread thread;
    private Background bg;
    private GameObject player;
    private Foe foe;
    private Maze maze;
    private LayerManager layerManager;

    private Vector inputListeners;
    private Camera camera;

    /*
     *  Constructor: must call super(boolean)
     */
    public MyCanvas(){
        super(true);    // true = suppress sending key events
    }

    public void start(){
        inputListeners = new Vector();
        viewCoords = new Coordinates(0, 0);
        maxScreenCoords = new Coordinates(200, 200);
        camera = new Camera(getWidth(), getHeight(), viewCoords, maxScreenCoords);
        layerManager = new LayerManager();
        bg = new Background();
        maze = new Maze(maxScreenCoords);
        Coordinates coordinates = new Coordinates(29, 169);
        player = new Player(coordinates);
        foe = new Foe((new Coordinates(160, 20)), 1, player);
        try{
            thread = new Thread(this);
            thread.start();
        }
        catch(Exception e){
            System.out.println("Error loading sprite!!");
        }
        inputListeners.addElement(player);
        foe.addToLayerManager(layerManager);
        player.addToLayerManager(layerManager);
        maze.addToLayerManager(layerManager);
        bg.addToLayerManager(layerManager);
    }

   /*
    * Method to draw the updated display
    */
   public void gameDraw(Graphics g){
       bg.update();
       layerManager.paint(g, viewCoords.getX(), viewCoords.getY());
       flushGraphics();
   }

   public void stop(){
       running = false;
   }
   
   /*
    * The game loop polls for key states and updates the
    * sprite's position
    */
   public void run(){
       Graphics g = getGraphics();
       while(true){
           for (int i = 0; i < inputListeners.size(); i++){
                InputListener listener = (InputListener) inputListeners.elementAt(i);
                listener.acceptInput(getKeyStates());
           }
           camera.trackGameObject(player);
           foe.chase();
           Vector collided = CollisionDetector.getTheCollisionDetector().detect();
           for (int i = 0; i < collided.size(); i++){
               Collision collision = (Collision) collided.elementAt(i);
               collision.resolve();
           }
           gameDraw(g);
           try{
                Thread.sleep(30);
           }
           catch(InterruptedException e){}
       }
    }
 }